"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

/*
  Copyright 2018 Google LLC

  Use of this source code is governed by an MIT-style
  license that can be found in the LICENSE file or at
  https://opensource.org/licenses/MIT.
*/
const _require = require('webpack-sources'),
      ConcatSource = _require.ConcatSource;

const SingleEntryPlugin = require('webpack/lib/SingleEntryPlugin');

const path = require('path');

const validate = require('workbox-build/build/entry-points/options/validate');

const webpackInjectManifestSchema = require('workbox-build/build/entry-points/options/webpack-inject-manifest-schema');

const getManifestEntriesFromCompilation = require('./lib/get-manifest-entries-from-compilation');

const relativeToOutputPath = require('./lib/relative-to-output-path');

const stringifyManifest = require('./lib/stringify-manifest');
/**
 * This class supports compiling a service worker file provided via `swSrc`,
 * and injecting into that service worker a list of URLs and revision
 * information for precaching based on the webpack asset pipeline.
 *
 * Use an instance of `InjectManifest` in the
 * [`plugins` array](https://webpack.js.org/concepts/plugins/#usage) of a
 * webpack config.
 *
 * @module workbox-webpack-plugin
 */


class InjectManifest {
  /**
   * Creates an instance of InjectManifest.
   *
   * @param {Object} [config] See the
   * [configuration guide](https://developers.google.com/web/tools/workbox/modules/workbox-webpack-plugin#configuration)
   * for all supported options and defaults.
   */
  constructor(config = {}) {
    this.config = config;
  }
  /**
   * @param {Object} [compiler] default compiler object passed from webpack
   *
   * @private
   */


  propagateWebpackConfig(compiler) {
    // Because this.config is listed last, properties that are already set
    // there take precedence over derived properties from the compiler.
    this.config = Object.assign({
      mode: compiler.mode
    }, this.config);
  }
  /**
   * @param {Object} [compiler] default compiler object passed from webpack
   *
   * @private
   */


  apply(compiler) {
    this.propagateWebpackConfig(compiler);
    compiler.hooks.make.tapPromise(this.constructor.name, compilation => this.handleMake(compilation, compiler).catch(error => compilation.errors.push(error)));
    compiler.hooks.emit.tapPromise(this.constructor.name, compilation => this.handleEmit(compilation).catch(error => compilation.errors.push(error)));
  }
  /**
   * @param {Object} compilation The webpack compilation.
   * @param {Object} parentCompiler The webpack parent compiler.
   *
   * @private
   */


  handleMake(compilation, parentCompiler) {
    var _this = this;

    return (0, _asyncToGenerator2.default)(function* () {
      try {
        _this.config = validate(_this.config, webpackInjectManifestSchema);
      } catch (error) {
        throw new Error(`Please check your ${_this.constructor.name} plugin ` + `configuration:\n${error.message}`);
      }

      _this.config.swDest = relativeToOutputPath(compilation, _this.config.swDest);
      const outputOptions = {
        path: parentCompiler.options.output.path,
        filename: _this.config.swDest
      };
      const childCompiler = compilation.createChildCompiler(_this.constructor.name, outputOptions, _this.config.webpackCompilationPlugins);
      childCompiler.context = parentCompiler.context;
      childCompiler.inputFileSystem = parentCompiler.inputFileSystem;
      childCompiler.outputFileSystem = parentCompiler.outputFileSystem;
      new SingleEntryPlugin(parentCompiler.context, _this.config.swSrc, _this.constructor.name).apply(childCompiler);
      yield new Promise((resolve, reject) => {
        childCompiler.runAsChild((error, entries, childCompilation) => {
          if (error) {
            reject(error);
          } else {
            compilation.warnings = compilation.warnings.concat(childCompilation.warnings);
            compilation.errors = compilation.errors.concat(childCompilation.errors);
            resolve();
          }
        });
      });
    })();
  }
  /**
   * @param {Object} compilation The webpack compilation.
   *
   * @private
   */


  handleEmit(compilation) {
    var _this2 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      // See https://webpack.js.org/contribute/plugin-patterns/#monitoring-the-watch-graph
      const absoluteSwSrc = path.resolve(_this2.config.swSrc);
      compilation.fileDependencies.add(absoluteSwSrc);
      const swAsset = compilation.assets[_this2.config.swDest];
      delete compilation.assets[_this2.config.swDest];
      const manifestEntries = getManifestEntriesFromCompilation(compilation, _this2.config);
      const manifestDeclaration = stringifyManifest(manifestEntries, _this2.config.injectionPoint, _this2.config.mode !== 'production');
      compilation.assets[_this2.config.swDest] = new ConcatSource(manifestDeclaration, swAsset || '');
    })();
  }

}

module.exports = InjectManifest;