"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

/*
  Copyright 2018 Google LLC

  Use of this source code is governed by an MIT-style
  license that can be found in the LICENSE file or at
  https://opensource.org/licenses/MIT.
*/
const _require = require('webpack/lib/ModuleFilenameHelpers'),
      matchPart = _require.matchPart;

const transformManifest = require('workbox-build/build/lib/transform-manifest');

const getAssetHash = require('./get-asset-hash');

const resolveWebpackURL = require('./resolve-webpack-url');
/**
 * For a given asset, checks whether at least one of the conditions matches.
 *
 * @param {Asset} asset The webpack asset in question. This will be passed
 * to any functions that are listed as conditions.
 * @param {Compilation} compilation The webpack compilation. This will be passed
 * to any functions that are listed as conditions.
 * @param {Array<string|RegExp|Function>} conditions
 * @return {boolean} Whether or not at least one condition matches.
 * @private
 */


function checkConditions(asset, compilation, conditions = []) {
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = conditions[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      const condition = _step.value;

      if (typeof condition === 'function') {
        if (condition({
          asset,
          compilation
        })) {
          return true;
        }
      } else {
        // See https://github.com/webpack/webpack/blob/bf3e869a423a60581dcb64e215b8d14403e997f2/lib/ModuleFilenameHelpers.js#L151-L159
        if (matchPart(asset.name, condition)) {
          return true;
        }
      }
    } // We'll only get here if none of the conditions applied.

  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return false;
}
/**
 * Creates a mapping of an asset name to an Set of zero or more chunk names
 * that the asset is associated with.
 *
 * Those chunk names come from a combination of the `chunkName` property on the
 * asset, as well as the `stats.namedChunkGroups` property. That is the only
 * way to find out if an asset has an implicit descendent relationship with a
 * chunk, if it was, e.g., created by `SplitChunksPlugin`.
 *
 * See https://github.com/GoogleChrome/workbox/issues/1859
 * See https://github.com/webpack/webpack/issues/7073
 *
 * @param {Object} stats The webpack compilation stats.
 * @return {Object<string,Set<string>>}
 * @private
 */


function assetToChunkNameMapping(stats) {
  const mapping = {};
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = stats.assets[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      const asset = _step2.value;
      mapping[asset.name] = new Set(asset.chunkNames);
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  var _arr = Object.entries(stats.namedChunkGroups);

  for (var _i = 0; _i < _arr.length; _i++) {
    const _arr$_i = (0, _slicedToArray2.default)(_arr[_i], 2),
          chunkName = _arr$_i[0],
          assets = _arr$_i[1].assets;

    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = assets[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        const assetName = _step3.value;
        mapping[assetName].add(chunkName);
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }
  }

  return mapping;
}
/**
 * Filters the set of assets out, based on the configuration options provided:
 * - chunks and excludeChunks, for chunkName-based criteria.
 * - include and exclude, for more general criteria.
 *
 * @param {Compilation} compilation The webpack compilation.
 * @param {Object} config The validated configuration, obtained from the plugin.
 * @return {Set<Asset>} The assets that should be included in the manifest,
 * based on the criteria provided.
 * @private
 */


function filterAssets(compilation, config) {
  const filteredAssets = new Set();
  const stats = compilation.getStats().toJson();
  const assetNameToChunkNames = assetToChunkNameMapping(stats); // See https://github.com/GoogleChrome/workbox/issues/1287

  if (Array.isArray(config.chunks)) {
    var _iteratorNormalCompletion4 = true;
    var _didIteratorError4 = false;
    var _iteratorError4 = undefined;

    try {
      for (var _iterator4 = config.chunks[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
        const chunk = _step4.value;

        if (!(chunk in stats.namedChunkGroups)) {
          compilation.warnings.push(`The chunk '${chunk}' was provided in ` + `your Workbox chunks config, but was not found in the compilation.`);
        }
      }
    } catch (err) {
      _didIteratorError4 = true;
      _iteratorError4 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion4 && _iterator4.return != null) {
          _iterator4.return();
        }
      } finally {
        if (_didIteratorError4) {
          throw _iteratorError4;
        }
      }
    }
  } // See https://webpack.js.org/api/stats/#asset-objects


  var _iteratorNormalCompletion5 = true;
  var _didIteratorError5 = false;
  var _iteratorError5 = undefined;

  try {
    for (var _iterator5 = stats.assets[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
      const asset = _step5.value;
      // chunkName based filtering is funky because:
      // - Each asset might belong to one or more chunkNames.
      // - If *any* of those chunk names match our config.excludeChunks,
      //   then we skip that asset.
      // - If the config.chunks is defined *and* there's no match
      //   between at least one of the chunkNames and one entry, then
      //   we skip that assets as well.
      const isExcludedChunk = Array.isArray(config.excludeChunks) && config.excludeChunks.some(chunkName => {
        return assetNameToChunkNames[asset.name].has(chunkName);
      });

      if (isExcludedChunk) {
        continue;
      }

      const isIncludedChunk = !Array.isArray(config.chunks) || config.chunks.some(chunkName => {
        return assetNameToChunkNames[asset.name].has(chunkName);
      });

      if (!isIncludedChunk) {
        continue;
      } // Next, check asset-level checks via includes/excludes:


      const isExcluded = checkConditions(asset, compilation, config.exclude);

      if (isExcluded) {
        continue;
      } // Treat an empty config.includes as an implicit inclusion.


      const isIncluded = !Array.isArray(config.include) || checkConditions(asset, compilation, config.include);

      if (!isIncluded) {
        continue;
      } // If we've gotten this far, then add the asset.


      filteredAssets.add(asset);
    }
  } catch (err) {
    _didIteratorError5 = true;
    _iteratorError5 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion5 && _iterator5.return != null) {
        _iterator5.return();
      }
    } finally {
      if (_didIteratorError5) {
        throw _iteratorError5;
      }
    }
  }

  return filteredAssets;
}

module.exports = (compilation, config) => {
  const filteredAssets = filterAssets(compilation, config);
  const publicPath = compilation.options.output.publicPath;
  const fileDetails = [];
  var _iteratorNormalCompletion6 = true;
  var _didIteratorError6 = false;
  var _iteratorError6 = undefined;

  try {
    for (var _iterator6 = filteredAssets[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
      const asset = _step6.value;

      // Not sure why this would be false, but checking just in case, since
      // our original list of assets comes from compilation.getStats().toJson(),
      // not from compilation.assets.
      if (asset.name in compilation.assets) {
        // This matches the format expected by transformManifest().
        fileDetails.push({
          file: resolveWebpackURL(publicPath, asset.name),
          hash: getAssetHash(compilation.assets[asset.name]),
          size: asset.size || 0
        });
      } else {
        compilation.warnings.push(`Could not precache ${asset.name}, as it's ` + `missing from compilation.assets. Please open a bug against Workbox ` + `with details about your webpack config.`);
      }
    } // We also get back `size` and `count`, and it would be nice to log that
    // somewhere, but... webpack doesn't offer info-level logs?
    // https://github.com/webpack/webpack/issues/3996

  } catch (err) {
    _didIteratorError6 = true;
    _iteratorError6 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion6 && _iterator6.return != null) {
        _iterator6.return();
      }
    } finally {
      if (_didIteratorError6) {
        throw _iteratorError6;
      }
    }
  }

  const _transformManifest = transformManifest({
    dontCacheBustURLsMatching: config.dontCacheBustURLsMatching,
    manifestTransforms: config.manifestTransforms,
    maximumFileSizeToCacheInBytes: config.maximumFileSizeToCacheInBytes,
    modifyURLPrefix: config.modifyURLPrefix,
    transformParam: compilation,
    fileDetails
  }),
        manifestEntries = _transformManifest.manifestEntries,
        warnings = _transformManifest.warnings;

  compilation.warnings = compilation.warnings.concat(warnings || []); // Ensure that the entries are properly sorted by URL.

  const sortedEntries = manifestEntries.sort((a, b) => a.url === b.url ? 0 : a.url > b.url ? 1 : -1);
  return sortedEntries;
};